# -*- coding: utf-8 -*-

# Name of the environment variable with the API key
ENV_VARIABLE = 'SCRAPINGHUB_AUTOEXTRACT_KEY'

# Endpoint
API_ENDPOINT = 'https://autoextract.scrapinghub.com/v1/extract'

# Default timeout that server uses. Client timeouts should be larger than that.
API_TIMEOUT = 600

# Max batch size
API_MAX_BATCH = 100
